#define PLATFORM_WCS                                                                                                             \
	"*********** PLATFORM POSITION VECTOR **********\n"                                                                          \
	"reserved1			==>	  %.128s\n"                                                                                                 \
	"num_data_points			==>	  %.4s\n"                                                                                             \
	"year_of_data_points		==>	  %.4s\n"                                                                                          \
	"month_of_data_points		==>	  %.4s\n"                                                                                         \
	"day_of_data_points		==>	  %.4s\n"                                                                                           \
	"day_of_data_points_in_year	==>	  %.4s\n"                                                                                    \
	"sec_of_day_of_data		==>	  %.22s\n"                                                                                          \
	"data_points_time_gap		==>	  %.22s\n"                                                                                        \
	"ref_coord_sys			==>	  %.64s\n"                                                                                              \
	"greenwhich_mean_hour_angle	==>	  %.22s\n"                                                                                   \
	"a_track_pos_err			==>	  %.16s\n"                                                                                            \
	"c_track_pos_err			==>	  %.16s\n"                                                                                            \
	"radial_pos_err			==>	  %.16s\n"                                                                                             \
	"reserved2			==>	  %.48s\n\n"

#define POSITION_VECTOR_WCS                                                                                                      \
	"*********** PLATFORM VECTOR **********\n"                                                                                   \
	"pos_x	==>        %.22s\n"                                                                                                   \
	"pos_y	==>        %.22s\n"                                                                                                   \
	"pos_z	==>        %.22s\n"                                                                                                   \
	"vel_x	==>        %.22s\n"                                                                                                   \
	"vel_y	==>        %.22s\n"                                                                                                   \
	"vel_z	==>        %.22s\n\n"
