#define SARLEADER_DSS_WCS                                                                                                        \
	"*********** DSS RECORD ***********\n"                                                                                       \
	"dss_rec_seq_num  			==>	%.4s\n"                                                                                             \
	"chan_ind  				==>	%.4s\n"                                                                                                   \
	"reserved1  				==>	%.16s\n"                                                                                                 \
	"scene_number  				==>	%.32s\n"                                                                                              \
	"input_scene_center_time  		==>	%.32s\n"                                                                                     \
	"spare1  				==>	%.16s\n"                                                                                                    \
	"center_lat  				==>	%.16s\n"                                                                                                \
	"center_long  				==>	%.16s\n"                                                                                               \
	"center_heading  			==>	%.16s\n"                                                                                             \
	"ellipsoid_designator  			==>	%.16s\n"                                                                                       \
	"ellipsoid_semimajor_axis  		==>	%.16s\n"                                                                                    \
	"ellipsoid_semiminor_axis  		==>	%.16s\n"                                                                                    \
	"earth_constant  			==>	%.16s\n"                                                                                             \
	"spare2  				==>	%.16s\n"                                                                                                    \
	"ellipsoid_j2  				==>	%.16s\n"                                                                                              \
	"ellipsoid_j3  				==>	%.16s\n"                                                                                              \
	"ellipsoid_j4  				==>	%.16s\n"                                                                                              \
	"spare					==>	%.16s\n"                                                                                                      \
	"reserved_new  				==>	%.16s\n"                                                                                              \
	"scene_centre_line_number  		==>	%.8s\n"                                                                                     \
	"scene_centre_pixel_number  		==>	%.8s\n"                                                                                    \
	"scene_length  				==>	%.16s\n"                                                                                              \
	"scene_width  				==>	%.16s\n"                                                                                               \
	"spare3  				==>	%.16s\n"                                                                                                    \
	"nchan					==>	%.4s\n"                                                                                                       \
	"spare4  				==>	%.4s\n"                                                                                                     \
	"mission_identifier  			==>	%.16s\n"                                                                                         \
	"sensor_id_and_mode  			==>	%.32s\n"                                                                                         \
	"orbit_number  				==>	%.8s\n"                                                                                               \
	"lat_nadir_center  			==>	%.8s\n"                                                                                            \
	"long_nadir_center  			==>	%.8s\n"                                                                                           \
	"heading_nadir_center  			==>	%.8s\n"                                                                                        \
	"clock_angel  				==>	%.8s\n"                                                                                                \
	"incidence_angle_center  		==>	%.8s\n"                                                                                       \
	"radar_freq  				==>	%.8s\n"                                                                                                 \
	"radar_wavelength  			==>	%.16s\n"                                                                                           \
	"motion_compensation  			==>	%.2s\n"                                                                                         \
	"range_pulse_code_specifier  		==>	%.16s\n"                                                                                  \
	"range_pulse_amplitude_const  		==>	%.16s\n"                                                                                 \
	"range_pulse_amplitude_lin  		==>	%.16s\n"                                                                                   \
	"range_pulse_amplitude_quad  		==>	%.16s\n"                                                                                  \
	"range_pulse_amplitude_cube  		==>	%.16s\n"                                                                                  \
	"range_pulse_amplitude_quart  		==>	%.16s\n"                                                                                 \
	"range_pulse_phase_const  		==>	%.16s\n"                                                                                     \
	"range_pulse_phase_lin			==>	%.16s\n"                                                                                        \
	"range_pulse_phase_quad  		==>	%.16s\n"                                                                                      \
	"range_pulse_phase_cube  		==>	%.16s\n"                                                                                      \
	"range_pulse_phase_quart  		==>	%.16s\n"                                                                                     \
	"chirp_extraction_index  		==>	%.8s\n"                                                                                       \
	"spare5  				==>	%.8s\n"                                                                                                     \
	"sampling  				==>	%.16s\n"                                                                                                  \
	"range_gate_early_edge_start_image 	==>	%.16s\n"                                                                             \
	"range_pulse_length  			==>	%.16s\n"                                                                                         \
	"reserved2  				==>	%.4s\n"                                                                                                  \
	"range_compressed_flag			==>	%.4s\n"                                                                                         \
	"reserved3  				==>	%.32s\n"                                                                                                 \
	"quantisation_in_bits  			==>	%.8s\n"                                                                                        \
	"quantizer_descriptor  			==>	%.12s\n"                                                                                       \
	"dc_bias_i  				==>	%.16s\n"                                                                                                 \
	"dc_bias_q  				==>	%.16s\n"                                                                                                 \
	"gain_imbalance  			==>	%.16s\n"                                                                                             \
	"spare6  				==>	%.32s\n"                                                                                                    \
	"reserved4  				==>	%.16s\n"                                                                                                 \
	"antenna_mech_bor  			==>	%.16s\n"                                                                                           \
	"reserved5  				==>	%.4s\n"                                                                                                  \
	"nominal_prf  				==>	%.16s\n"                                                                                               \
	"reserved6  				==>	%.32s\n"                                                                                                 \
	"satelite_encoded_binary_time  		==>	%.16s\n"                                                                                \
	"satelite_clock_time  			==>	%.32s\n"                                                                                        \
	"satelite_clock_increment  		==>	%.8s\n"                                                                                     \
	"spare7  				==>	%.8s\n"                                                                                                     \
	"processing_facility_identifier		==>	%.16s\n"                                                                                \
	"processing_system_id  			==>	%.8s\n"                                                                                        \
	"processing_version_id			==>	%.8s\n"                                                                                         \
	"reserved7  				==>	%.32s\n"                                                                                                 \
	"product_type_id  			==>	%.32s\n"                                                                                            \
	"alg_id  				==>	%.32s\n"                                                                                                    \
	"nlooks_az  				==>	%.16s\n"                                                                                                 \
	"neff_looks_range  			==>	%.16s\n"                                                                                           \
	"bandwidth_look_az			==>	%.16s\n"                                                                                            \
	"bandwidth_look_range  			==>	%.16s\n"                                                                                       \
	"total_look_bandwidth_az  		==>	%.16s\n"                                                                                     \
	"total_look_bandwidth_range  		==>	%.16s\n"                                                                                  \
	"w_func_designator_az  			==>	%.32s\n"                                                                                       \
	"w_func_designator_range  		==>	%.32s\n"                                                                                     \
	"data_input_source  			==>	%.16s\n"                                                                                          \
	"nom_res_3db_range  			==>	%.16s\n"                                                                                          \
	"nom_res_az				==>	%.16s\n"                                                                                                  \
	"reserved8				==>	%.32s\n"                                                                                                   \
	"a_track_dop_freq_const_early_image  	==>	%.16s\n"                                                                           \
	"a_track_dop_freq_lin_early_image  	==>	%.16s\n"                                                                             \
	"a_track_dop_freq_quad_early_image  	==>	%.16s\n"                                                                            \
	"spare8					==>	%.16s\n"                                                                                                     \
	"c_track_dop_freq_const_early_image  	==>	%.16s\n"                                                                           \
	"c_track_dop_freq_lin_early_image  	==>	%.16s\n"                                                                             \
	"c_track_dop_freq_quad_early_image  	==>	%.16s\n"                                                                            \
	"time_direction_along_pixel  		==>	%.8s\n"                                                                                   \
	"time_direction_along_line  		==>	%.8s\n"                                                                                    \
	"a_track_dop_freq_rate_const_early_image	==>	%.16s\n"                                                                        \
	"a_track_dop_freq_rate_lin_early_image  	==>	%.16s\n"                                                                        \
	"a_track_dop_freq_rate_quad_early_image 	==>	%.16s\n"                                                                        \
	"spare9					==>	%.16s\n"                                                                                                     \
	"c_track_dop_freq_rate_const_early_image	==>	%.16s\n"                                                                        \
	"c_track_dop_freq_rate_lin_early_image	==>	%.16s\n"                                                                          \
	"c_track_dop_freq_rate_quad_early_image	==>	%.16s\n"                                                                         \
	"spare10					==>	%.16s\n"                                                                                                    \
	"line_content_indicator  		==>	%.8s\n"                                                                                       \
	"clut_lock_flag  			==>	%.4s\n"                                                                                              \
	"autofocussing_flag			==>	%.4s\n"                                                                                            \
	"line_spacing  				==>	%.16s\n"                                                                                              \
	"pixel_spacing_range  			==>	%.16s\n"                                                                                        \
	"range_compression_designator  		==>	%.16s\n"                                                                                \
	"spare11  				==>	%.32s\n"                                                                                                   \
	"zero_dop_range_time_f_pixel  		==>	%.16s\n"                                                                                 \
	"zero_dop_range_time_c_pixel  		==>	%.16s\n"                                                                                 \
	"zero_dop_range_time_l_pixel  		==>	%.16s\n"                                                                                 \
	"zero_dop_az_time_f_pixel  		==>	%.24s\n"                                                                                    \
	"zero_dop_az_time_c_pixel  		==>	%.24s\n"                                                                                    \
	"zero_dop_az_time_l_pixel  		==>	%.24s\n"                                                                                    \
	"gec_local_use_segment			==>	%.546s\n\n"

#define SARLEADER_DPAF_DSS_WCS                                                                                                   \
	"*********** DSS RECORD ***********\n"                                                                                       \
	"dss_rec_seq_num  			==>	%.4s\n"                                                                                             \
	"chan_ind  				==>	%.4s\n"                                                                                                   \
	"reserved1  				==>	%.16s\n"                                                                                                 \
	"scene_number  				==>	%.32s\n"                                                                                              \
	"input_scene_center_time  		==>	%.32s\n"                                                                                     \
	"spare1  				==>	%.16s\n"                                                                                                    \
	"center_lat  				==>	%.16s\n"                                                                                                \
	"center_long  				==>	%.16s\n"                                                                                               \
	"center_heading  			==>	%.16s\n"                                                                                             \
	"ellipsoid_designator  			==>	%.16s\n"                                                                                       \
	"ellipsoid_semimajor_axis  		==>	%.16s\n"                                                                                    \
	"ellipsoid_semiminor_axis  		==>	%.16s\n"                                                                                    \
	"earth_constant  			==>	%.16s\n"                                                                                             \
	"spare2  				==>	%.16s\n"                                                                                                    \
	"ellipsoid_j2  				==>	%.16s\n"                                                                                              \
	"ellipsoid_j3  				==>	%.16s\n"                                                                                              \
	"ellipsoid_j4  				==>	%.16s\n"                                                                                              \
	"spare					==>	%.16s\n"                                                                                                      \
	"reserved_new  				==>	%.16s\n"                                                                                              \
	"scene_centre_line_number  		==>	%.8s\n"                                                                                     \
	"scene_centre_pixel_number  		==>	%.8s\n"                                                                                    \
	"scene_length  				==>	%.16s\n"                                                                                              \
	"scene_width  				==>	%.16s\n"                                                                                               \
	"spare3  				==>	%.16s\n"                                                                                                    \
	"nchan					==>	%.4s\n"                                                                                                       \
	"spare4  				==>	%.4s\n"                                                                                                     \
	"mission_identifier  			==>	%.16s\n"                                                                                         \
	"sensor_id_and_mode  			==>	%.32s\n"                                                                                         \
	"orbit_number  				==>	%.8s\n"                                                                                               \
	"lat_nadir_center  			==>	%.8s\n"                                                                                            \
	"long_nadir_center  			==>	%.8s\n"                                                                                           \
	"heading_nadir_center  			==>	%.8s\n"                                                                                        \
	"clock_angel  				==>	%.8s\n"                                                                                                \
	"incidence_angle_center  		==>	%.8s\n"                                                                                       \
	"radar_freq  				==>	%.8s\n"                                                                                                 \
	"radar_wavelength  			==>	%.16s\n"                                                                                           \
	"motion_compensation  			==>	%.2s\n"                                                                                         \
	"range_pulse_code_specifier  		==>	%.16s\n"                                                                                  \
	"range_pulse_amplitude_const  		==>	%.16s\n"                                                                                 \
	"range_pulse_amplitude_lin  		==>	%.16s\n"                                                                                   \
	"range_pulse_amplitude_quad  		==>	%.16s\n"                                                                                  \
	"range_pulse_amplitude_cube  		==>	%.16s\n"                                                                                  \
	"range_pulse_amplitude_quart  		==>	%.16s\n"                                                                                 \
	"range_pulse_phase_const  		==>	%.16s\n"                                                                                     \
	"range_pulse_phase_lin			==>	%.16s\n"                                                                                        \
	"range_pulse_phase_quad  		==>	%.16s\n"                                                                                      \
	"range_pulse_phase_cube  		==>	%.16s\n"                                                                                      \
	"range_pulse_phase_quart  		==>	%.16s\n"                                                                                     \
	"chirp_extraction_index  		==>	%.8s\n"                                                                                       \
	"spare5  				==>	%.8s\n"                                                                                                     \
	"sampling  				==>	%.16s\n"                                                                                                  \
	"range_gate_early_edge_start_image 	==>	%.16s\n"                                                                             \
	"range_pulse_length  			==>	%.16s\n"                                                                                         \
	"reserved2  				==>	%.4s\n"                                                                                                  \
	"range_compressed_flag			==>	%.4s\n"                                                                                         \
	"reserved3  				==>	%.32s\n"                                                                                                 \
	"quantisation_in_bits  			==>	%.8s\n"                                                                                        \
	"quantizer_descriptor  			==>	%.12s\n"                                                                                       \
	"dc_bias_i  				==>	%.16s\n"                                                                                                 \
	"dc_bias_q  				==>	%.16s\n"                                                                                                 \
	"gain_imbalance  			==>	%.16s\n"                                                                                             \
	"spare6  				==>	%.32s\n"                                                                                                    \
	"reserved4  				==>	%.16s\n"                                                                                                 \
	"antenna_mech_bor  			==>	%.16s\n"                                                                                           \
	"reserved5  				==>	%.4s\n"                                                                                                  \
	"nominal_prf  				==>	%.16s\n"                                                                                               \
	"reserved6  				==>	%.32s\n"                                                                                                 \
	"satelite_encoded_binary_time  		==>	%.16s\n"                                                                                \
	"satelite_clock_time  			==>	%.32s\n"                                                                                        \
	"satelite_clock_increment  		==>	%.8s\n"                                                                                     \
	"spare7  				==>	%.8s\n"                                                                                                     \
	"processing_facility_identifier		==>	%.16s\n"                                                                                \
	"processing_system_id  			==>	%.8s\n"                                                                                        \
	"processing_version_id			==>	%.8s\n"                                                                                         \
	"reserved7  				==>	%.32s\n"                                                                                                 \
	"product_type_id  			==>	%.32s\n"                                                                                            \
	"alg_id  				==>	%.32s\n"                                                                                                    \
	"nlooks_az  				==>	%.16s\n"                                                                                                 \
	"neff_looks_range  			==>	%.16s\n"                                                                                           \
	"bandwidth_look_az			==>	%.16s\n"                                                                                            \
	"bandwidth_look_range  			==>	%.16s\n"                                                                                       \
	"total_look_bandwidth_az  		==>	%.16s\n"                                                                                     \
	"total_look_bandwidth_range  		==>	%.16s\n"                                                                                  \
	"w_func_designator_az  			==>	%.32s\n"                                                                                       \
	"w_func_designator_range  		==>	%.32s\n"                                                                                     \
	"data_input_source  			==>	%.16s\n"                                                                                          \
	"nom_res_3db_range  			==>	%.16s\n"                                                                                          \
	"nom_res_az				==>	%.16s\n"                                                                                                  \
	"reserved8				==>	%.32s\n"                                                                                                   \
	"a_track_dop_freq_const_early_image  	==>	%.16s\n"                                                                           \
	"a_track_dop_freq_lin_early_image  	==>	%.16s\n"                                                                             \
	"a_track_dop_freq_quad_early_image  	==>	%.16s\n"                                                                            \
	"spare8					==>	%.16s\n"                                                                                                     \
	"c_track_dop_freq_const_early_image  	==>	%.16s\n"                                                                           \
	"c_track_dop_freq_lin_early_image  	==>	%.16s\n"                                                                             \
	"c_track_dop_freq_quad_early_image  	==>	%.16s\n"                                                                            \
	"time_direction_along_pixel  		==>	%.8s\n"                                                                                   \
	"time_direction_along_line  		==>	%.8s\n"                                                                                    \
	"a_track_dop_freq_rate_const_early_image	==>	%.16s\n"                                                                        \
	"a_track_dop_freq_rate_lin_early_image  	==>	%.16s\n"                                                                        \
	"a_track_dop_freq_rate_quad_early_image 	==>	%.16s\n"                                                                        \
	"spare9					==>	%.16s\n"                                                                                                     \
	"c_track_dop_freq_rate_const_early_image	==>	%.16s\n"                                                                        \
	"c_track_dop_freq_rate_lin_early_image	==>	%.16s\n"                                                                          \
	"c_track_dop_freq_rate_quad_early_image	==>	%.16s\n"                                                                         \
	"spare10					==>	%.16s\n"                                                                                                    \
	"line_content_indicator  		==>	%.8s\n"                                                                                       \
	"clut_lock_flag  			==>	%.4s\n"                                                                                              \
	"autofocussing_flag			==>	%.4s\n"                                                                                            \
	"line_spacing  				==>	%.16s\n"                                                                                              \
	"pixel_spacing_range  			==>	%.16s\n"                                                                                        \
	"range_compression_designator  		==>	%.16s\n"                                                                                \
	"spare11  				==>	%.32s\n"                                                                                                   \
	"zero_dop_range_time_f_pixel  		==>	%.16s\n"                                                                                 \
	"zero_dop_range_time_c_pixel  		==>	%.16s\n"                                                                                 \
	"zero_dop_range_time_l_pixel  		==>	%.16s\n"                                                                                 \
	"zero_dop_az_time_f_pixel  		==>	%.24s\n"                                                                                    \
	"zero_dop_az_time_c_pixel  		==>	%.24s\n"                                                                                    \
	"zero_dop_az_time_l_pixel  		==>	%.24s\n\n"
